@@ -1,8 +1,10 @@
 try:
     from unittest import mock
+    from urllib import parse
     MOCK_CLASS_NAME = 'unittest.mock.Mock'
 except ImportError:
     import mock
+    import urlparse as parse
     MOCK_CLASS_NAME = 'mock.Mock'
 
 import datetime
@@ -17,6 +19,8 @@ from facebook_auth.backends import UserFactory
 from facebook_auth import forms
 from facebook_auth import graph_api
 from facebook_auth import models
+from facebook_auth import urls
+from facebook_auth import views
 
 
 class TruncaterTest(test.SimpleTestCase):
@@ -82,13 +86,14 @@ class UserFactoryTest(test.TestCase):
 
 
 class UserFactoryOnErrorTest(test.TestCase):
-    def test(self):
+    def test_success(self):
         factory = UserFactory()
         factory.graph_api_class = mock.Mock()
         factory.graph_api_class.return_value.get.return_value = {'id': '123'}
         user = factory.get_user("123")
         self.assertEqual(123, user.user_id)
 
+    def test_success_in_retry(self):
         factory = UserFactory()
         factory.graph_api_class = mock.Mock()
         factory.graph_api_class.return_value.get.side_effect = [
@@ -99,6 +104,7 @@ class UserFactoryOnErrorTest(test.TestCase):
         user = factory.get_user("123")
         self.assertEqual(123, user.user_id)
 
+    def test_failure(self):
         factory = UserFactory()
         factory.graph_api_class = mock.Mock()
         factory.graph_api_class.return_value.get.side_effect = [
@@ -106,7 +112,8 @@ class UserFactoryOnErrorTest(test.TestCase):
             FacebookError("msg", 1),
             FacebookError("msg", 1),
             {'id': '123'}]
-        self.assertEqual(None, factory.get_user("123"))
+        with self.assertRaises(FacebookError):
+            factory.get_user("123")
 
 
 @mock.patch('django.utils.timezone.now')
@@ -288,3 +295,39 @@ class TestDebugAllTokensForUser(test.TestCase):
         models.debug_all_tokens_for_user('123')
         self.assertRaises(models.UserToken.DoesNotExist,
                           token_manager.get_access_token, '123')
+
+
+class TestNextUrl(test.TestCase):
+    def test_invalid_next(self):
+        with self.assertRaises(urls.InvalidNextUrl):
+            urls.Next().decode('1:2:3')
+
+    def test_invalid_next_format(self):
+        with self.assertRaises(urls.InvalidNextUrl):
+            urls.Next().decode('this is not valid signature')
+
+    def test_empty_next(self):
+        with self.assertRaises(urls.InvalidNextUrl):
+            urls.Next().decode('')
+
+
+class HandlerAcceptanceTest(test.TestCase):
+    @mock.patch('facebook_auth.views.authenticate')
+    def test_valid_next(self, authenticate):
+        authenticate.return_value = None
+        encoded_next = urls.Next().encode({
+            'next': 'http://next.example.com',
+            'close': 'http://close.example.com'})
+        next_value = parse.parse_qs(encoded_next)['next'][0]
+        request = mock.Mock(GET={'next': next_value, 'code': 'code'},
+                            method='GET')
+        with self.settings(FACEBOOK_CANVAS_URL='http://example.com'):
+            response = views.handler(request)
+        self.assertEqual(302, response.status_code)
+        self.assertEqual('http://next.example.com', response['Location'])
+
+    def test_invalid_next(self):
+        request = mock.Mock(GET={'next': 'a:b:c', 'code': 'code'},
+                            method='GET')
+        response = views.handler(request)
+        self.assertEqual(400, response.status_code)
